@@ -13,9 +13,7 @@ We can also move the implementation to C++ if desired.
 """
 
 import datetime
-import inspect
 import mimetypes
-import platform
 import shutil
 from collections.abc import KeysView, MutableMapping
 from decimal import Decimal
@@ -24,7 +22,6 @@ from pathlib import Path
 from subprocess import run
 from tempfile import NamedTemporaryFile
 from typing import (
-    Any,
     BinaryIO,
     Callable,
     ItemsView,
@@ -32,7 +29,6 @@ from typing import (
     List,
     Optional,
     Tuple,
-    Type,
     TypeVar,
     Union,
     ValuesView,
@@ -40,6 +36,7 @@ from typing import (
 from warnings import warn
 
 from . import Array, Dictionary, Name, Object, Page, Pdf, Stream
+from ._augments import augment_override_cpp, augments
 from ._qpdf import (
     AccessMode,
     AttachedFile,
@@ -64,127 +61,6 @@ __all__ = []
 Numeric = TypeVar('Numeric', int, float, Decimal)
 
 
-def augment_override_cpp(fn):
-    """This Python function should replace the C++ implementation, if there is one."""
-    fn._augment_override_cpp = True
-    return fn
-
-
-def augment_if_no_cpp(fn):
-    """This Python function provides a Python implementation if no C++ implementation exists."""
-    fn._augment_if_no_cpp = True
-    return fn
-
-
-def _is_inherited_method(meth):
-    # Augmenting a C++ with a method that cls inherits from the Python
-    # object is never what we want.
-    return meth.__qualname__.startswith('object.')
-
-
-def _is_augmentable(m):
-    return (
-        inspect.isfunction(m) and not _is_inherited_method(m)
-    ) or inspect.isdatadescriptor(m)
-
-
-def augments(cls_cpp: Type[Any]):
-    """Attach methods of a Python support class to an existing class
-
-    This monkeypatches all methods defined in the support class onto an
-    existing class. Example:
-
-    .. code-block:: python
-
-        @augments(ClassDefinedInCpp)
-        class SupportClass:
-            def foo(self):
-                pass
-
-    The Python method 'foo' will be monkeypatched on ClassDefinedInCpp. SupportClass
-    has no meaning on its own and should not be used, but gets returned from
-    this function so IDE code inspection doesn't get too confused.
-
-    We don't subclass because it's much more convenient to monkeypatch Python
-    methods onto the existing Python binding of the C++ class. For one thing,
-    this allows the implementation to be moved from Python to C++ or vice
-    versa. It saves having to implement an intermediate Python subclass and then
-    ensures that the C++ superclass never 'leaks' to pikepdf users. Finally,
-    wrapper classes and subclasses can become problematic if the call stack
-    crosses the C++/Python boundary multiple times.
-
-    Any existing methods may be used, regardless of whether they are defined
-    elsewhere in the support class or in the target class.
-
-    For data fields to work, the target class must be
-    tagged ``py::dynamic_attr`` in pybind11.
-
-    Strictly, the target class does not have to be C++ or derived from pybind11.
-    This works on pure Python classes too.
-
-    THIS DOES NOT work for class methods.
-
-    (Alternative ideas: https://github.com/pybind/pybind11/issues/1074)
-    """
-
-    OVERRIDE_WHITELIST = {'__eq__', '__hash__', '__repr__'}
-    if platform.python_implementation() == 'PyPy':
-        # Either PyPy or pybind11's interface to PyPy automatically adds a __getattr__
-        OVERRIDE_WHITELIST |= {'__getattr__'}  # pragma: no cover
-
-    def class_augment(cls, cls_cpp=cls_cpp):
-
-        # inspect.getmembers has different behavior on PyPy - in particular it seems
-        # that a typical PyPy class like cls will have more methods that it considers
-        # methods than CPython does. Our predicate should take care of this.
-        for name, member in inspect.getmembers(cls, predicate=_is_augmentable):
-            if name == '__weakref__':
-                continue
-            if (
-                hasattr(cls_cpp, name)
-                and hasattr(cls, name)
-                and name not in getattr(cls, '__abstractmethods__', set())
-                and name not in OVERRIDE_WHITELIST
-                and not getattr(getattr(cls, name), '_augment_override_cpp', False)
-            ):
-                if getattr(getattr(cls, name), '_augment_if_no_cpp', False):
-                    # If tagged as "augment if no C++", we are supported older pybind11
-                    # versions that may not define some functions. If pybind11 provides
-                    # the required function, don't override it.
-                    continue
-
-                # If the original C++ class and Python support class both define the
-                # same name, we generally have a conflict, because this is augmentation
-                # not inheritance. However, if the method provided by the support class
-                # is an abstract method, then we can consider the C++ version the
-                # implementation. Also, pybind11 provides defaults for __eq__,
-                # __hash__ and __repr__ that we often do want to override directly.
-
-                raise RuntimeError(  # pragma: no cover
-                    f"C++ {cls_cpp} and Python {cls} both define the same "
-                    f"non-abstract method {name}: "
-                    f"{getattr(cls_cpp, name, '')!r}, "
-                    f"{getattr(cls, name, '')!r}"
-                )
-            if inspect.isfunction(member):
-                setattr(cls_cpp, name, member)
-                installed_member = getattr(cls_cpp, name)
-                installed_member.__qualname__ = member.__qualname__.replace(
-                    cls.__name__, cls_cpp.__name__
-                )
-            elif inspect.isdatadescriptor(member):
-                setattr(cls_cpp, name, member)
-
-        def disable_init(self):
-            # Prevent initialization of the support class
-            raise NotImplementedError(self.__class__.__name__ + '.__init__')
-
-        cls.__init__ = disable_init
-        return cls
-
-    return class_augment
-
-
 def _single_page_pdf(page) -> bytes:
     """Construct a single page PDF from the provided page in memory."""
     pdf = Pdf.new()
@@ -265,10 +141,10 @@ class Extend_Object:
         for k in del_keys:
             del self[k]  # pylint: disable=unsupported-delete-operation
 
-    def _type_check_write(self, filter, decode_parms):
-        if isinstance(filter, list):
-            filter = Array(filter)
-        filter = filter.wrap_in_array()
+    def _type_check_write(self, filter_, decode_parms):
+        if isinstance(filter_, list):
+            filter_ = Array(filter_)
+        filter_ = filter_.wrap_in_array()
 
         if isinstance(decode_parms, list):
             decode_parms = Array(decode_parms)
@@ -277,7 +153,7 @@ class Extend_Object:
         else:
             decode_parms = decode_parms.wrap_in_array()
 
-        if not all(isinstance(item, Name) for item in filter):
+        if not all(isinstance(item, Name) for item in filter_):
             raise TypeError(
                 "filter must be: pikepdf.Name or pikepdf.Array([pikepdf.Name])"
             )
@@ -288,18 +164,18 @@ class Extend_Object:
                 "decode_parms must be: pikepdf.Dictionary or "
                 "pikepdf.Array([pikepdf.Dictionary])"
             )
-        if len(decode_parms) != 0 and len(filter) != len(decode_parms):
+        if len(decode_parms) != 0 and len(filter_) != len(decode_parms):
             raise ValueError(
-                f"filter ({repr(filter)}) and decode_parms "
+                f"filter ({repr(filter_)}) and decode_parms "
                 f"({repr(decode_parms)}) must be arrays of same length"
             )
-        if len(filter) == 1:
-            filter = filter[0]
+        if len(filter_) == 1:
+            filter_ = filter_[0]
         if len(decode_parms) == 0:
             decode_parms = None
         elif len(decode_parms) == 1:
             decode_parms = decode_parms[0]
-        return filter, decode_parms
+        return filter_, decode_parms
 
     def write(
         self,
@@ -943,14 +819,6 @@ class Extend_ObjectMapping:
         except KeyError:
             return default
 
-    @augment_if_no_cpp  # Remove this shim when dropping pybind11 < 2.8.0 support
-    def keys(self):
-        return KeysView(self)
-
-    @augment_if_no_cpp  # Remove this shim when dropping pybind11 < 2.8.0 support
-    def values(self):
-        return (v for _k, v in self.items())
-
 
 def check_is_box(obj) -> None:
     try:
@@ -1243,13 +1111,15 @@ class Extend_Page:
     @augment_override_cpp
     def __setattr__(self, name, value):
         if hasattr(self.__class__, name):
-            return object.__setattr__(self, name, value)
+            object.__setattr__(self, name, value)
+            return
         setattr(self.obj, name, value)
 
     @augment_override_cpp
     def __delattr__(self, name):
         if hasattr(self.__class__, name):
-            return object.__delattr__(self, name)
+            object.__delattr__(self, name)
+            return
         delattr(self.obj, name)
 
     def __getitem__(self, key):
@@ -1385,8 +1255,7 @@ class Extend_AttachedFileSpec:
                 f"<pikepdf._qpdf.AttachedFileSpec for {self.filename!r}, "
                 f"description {self.description!r}>"
             )
-        else:
-            return f"<pikepdf._qpdf.AttachedFileSpec description {self.description!r}>"
+        return f"<pikepdf._qpdf.AttachedFileSpec description {self.description!r}>"
 
 
 @augments(AttachedFile)
