@@ -11,11 +11,12 @@ from ._qpdf import pdf_doc_to_utf8, utf8_to_pdf_doc
 
 # pylint: disable=redefined-builtin
 
-# See PDF Reference Manual 1.7, Table D.2. We record every Unicode code point
-# that can be encoded to pdfdoc,
-# except for: [0x2D8, 0x2C7, 0x2C6, 0x2D9, 0x2DD, 0x2DB, 0x2DA, 0x2DC]
-# which qpdf does not seem to consider as encodable even though the PDF RM does.
-# Follow qpdf is more important for consistency.
+# See PDF Reference Manual 1.7, Table D.2.
+# The following generates set of all Unicode code points that can be encoded in
+# pdfdoc. Since pdfdoc is 8-bit, the vast majority of code points cannot be.
+#
+# Due to a bug, QPDF <= 10.5 and pikepdf < 5 had some inconsistencies around
+# PdfDocEncoding.
 PDFDOC_ENCODABLE = frozenset(
     list(range(0x00, 0x17 + 1))
     + list(range(0x20, 0x7E + 1))
@@ -51,9 +52,9 @@ PDFDOC_ENCODABLE = frozenset(
         0x0153,
         0x0161,
         0x017E,
-        0xFFFD,
         0x20AC,
     ]
+    + [0x02D8, 0x02C7, 0x02C6, 0x02D9, 0x02DD, 0x02DB, 0x02DA, 0x02DC]
     + list(range(0xA1, 0xAC + 1))
     + list(range(0xAE, 0xFF + 1))
 )
@@ -66,30 +67,13 @@ def _find_first_index(
         if is_whitelist and (ord(char) not in ordinals):
             return n
         if not is_whitelist and (ord(char) in ordinals):
-            return n
-    raise ValueError("couldn't find the unencodable character")
+            return n  # pragma: no cover
+    raise ValueError("couldn't find the unencodable character")  # pragma: no cover
 
 
 def pdfdoc_encode(input: str, errors: str = 'strict') -> Tuple[bytes, int]:
     error_marker = b'?' if errors == 'replace' else b'\xad'
-    try:
-        success, pdfdoc = utf8_to_pdf_doc(input, error_marker)
-    except RuntimeError as e:
-        # This error should occur from within in pybind11, with a RuntimeError
-        # message like:
-        #   "Unable to extract string contents! (encoding issue)"
-        # It is not known to happen except when str contains Unicode surrogates.
-        offending_index = _find_first_index(
-            input, range(0xD800, 0xDFFF + 1), is_whitelist=False
-        )
-
-        raise UnicodeEncodeError(
-            'pdfdoc',
-            input,
-            offending_index,
-            offending_index + 1,
-            "can't process Unicode surrogates",
-        ) from e
+    success, pdfdoc = utf8_to_pdf_doc(input, error_marker)
     if not success:
         if errors == 'strict':
             # libqpdf doesn't return what character caused the error, and Python
@@ -112,12 +96,23 @@ def pdfdoc_encode(input: str, errors: str = 'strict') -> Tuple[bytes, int]:
 def pdfdoc_decode(input: bytes, errors: str = 'strict') -> Tuple[str, int]:
     if isinstance(input, memoryview):
         input = input.tobytes()
-    utf8 = pdf_doc_to_utf8(input)
-    return utf8, len(input)
+    s = pdf_doc_to_utf8(input)
+    if errors == 'strict':
+        idx = s.find('\ufffd')
+        if idx >= 0:
+            raise UnicodeDecodeError(
+                'pdfdoc',
+                input,
+                idx,
+                idx + 1,
+                "no Unicode mapping is defined for this character",
+            )
+
+    return s, len(input)
 
 
 class PdfDocCodec(codecs.Codec):
-    """Implements PdfDocEncoding character map used inside PDFs"""
+    """Implements PdfDocEncoding character map used inside PDFs."""
 
     def encode(self, input: str, errors: str = 'strict') -> Tuple[bytes, int]:
         return pdfdoc_encode(input, errors)
@@ -131,16 +126,17 @@ class PdfDocStreamWriter(PdfDocCodec, codecs.StreamWriter):
 
 
 class PdfDocStreamReader(PdfDocCodec, codecs.StreamReader):
-    pass
+    def decode(self, input: bytes, errors: str = 'strict') -> Tuple[str, int]:
+        return PdfDocCodec.decode(self, input, errors)
 
 
 class PdfDocIncrementalEncoder(codecs.IncrementalEncoder):
-    def encode(self, input: str, final=False):
+    def encode(self, input: str, final: bool = False) -> bytes:
         return pdfdoc_encode(input, 'strict')[0]
 
 
 class PdfDocIncrementalDecoder(codecs.IncrementalDecoder):
-    def decode(self, input: bytes, final=False):
+    def decode(self, input: bytes, final: bool = False) -> str:
         return pdfdoc_decode(input, 'strict')[0]
 
 
