@@ -4,9 +4,16 @@
 #
 # Copyright (C) 2017, James R. Barlow (https://github.com/jbarlow83/)
 
-from typing import Any, Dict, NamedTuple, cast
+import sys
+from typing import TYPE_CHECKING, Any, Dict, NamedTuple, cast
 
-from pikepdf._qpdf import EncryptionMethod
+if sys.version_info >= (3, 8):
+    from typing import Literal
+else:
+    from typing_extensions import Literal  # pragma: no cover
+
+if TYPE_CHECKING:
+    from pikepdf._qpdf import EncryptionMethod
 
 
 class Permissions(NamedTuple):
@@ -37,7 +44,7 @@ class Permissions(NamedTuple):
     """Can users arrange document contents?"""
 
     modify_form: bool = True
-    """Can users fill out formts?"""
+    """Can users fill out forms?"""
 
     modify_other: bool = True
     """Can users modify the document?"""
@@ -83,19 +90,19 @@ class EncryptionInfo:
         return int(self._encdict['P'])
 
     @property
-    def stream_method(self) -> EncryptionMethod:
+    def stream_method(self) -> 'EncryptionMethod':
         """Encryption method used to encode streams."""
-        return cast(EncryptionMethod, self._encdict['stream'])
+        return cast('EncryptionMethod', self._encdict['stream'])
 
     @property
-    def string_method(self) -> EncryptionMethod:
+    def string_method(self) -> 'EncryptionMethod':
         """Encryption method used to encode strings."""
-        return cast(EncryptionMethod, self._encdict['string'])
+        return cast('EncryptionMethod', self._encdict['string'])
 
     @property
-    def file_method(self) -> EncryptionMethod:
+    def file_method(self) -> 'EncryptionMethod':
         """Encryption method used to encode the whole file."""
-        return cast(EncryptionMethod, self._encdict['file'])
+        return cast('EncryptionMethod', self._encdict['file'])
 
     @property
     def user_password(self) -> bytes:
@@ -124,44 +131,39 @@ class EncryptionInfo:
         return len(self._encdict['encryption_key']) * 8
 
 
-class Encryption(dict):
+class Encryption(NamedTuple):
     """
     Specify the encryption settings to apply when a PDF is saved.
-
-    Args:
-        owner: The owner password to use. This allows full control
-            of the file. If blank, the PDF will be encrypted and
-            present as "(SECURED)" in PDF viewers. If the owner password
-            is blank, the user password should be as well.
-        user: The user password to use. With this password, some
-            restrictions will be imposed by a typical PDF reader.
-            If blank, the PDF can be opened by anyone, but only modified
-            as allowed by the permissions in ``allow``.
-        R: Select the security handler algorithm to use. Choose from:
-            ``2``, ``3``, ``4`` or ``6``. By default, the highest version of
-            is selected (``6``). ``5`` is a deprecated algorithm that should
-            not be used.
-        allow: The permissions to set.
-            If omitted, all permissions are granted to the user.
-        aes: If True, request the AES algorithm. If False, use RC4.
-            If omitted, AES is selected whenever possible (R >= 4).
-        metadata: If True, also encrypt the PDF metadata. If False,
-            metadata is not encrypted. Reading document metadata without
-            decryption may be desirable in some cases. Requires ``aes=True``.
-            If omitted, metadata is encrypted whenever possible.
     """
 
-    def __init__(
-        self,
-        *,
-        owner: str,
-        user: str,
-        R: int = 6,
-        allow: Permissions = DEFAULT_PERMISSIONS,
-        aes: bool = True,
-        metadata: bool = True,
-    ):
-        super().__init__()
-        self.update(
-            dict(R=R, owner=owner, user=user, allow=allow, aes=aes, metadata=metadata)
-        )
+    owner: str = ''
+    """The owner password to use. This allows full control
+    of the file. If blank, the PDF will be encrypted and
+    present as "(SECURED)" in PDF viewers. If the owner password
+    is blank, the user password should be as well."""
+
+    user: str = ''
+    """The user password to use. With this password, some
+    restrictions will be imposed by a typical PDF reader.
+    If blank, the PDF can be opened by anyone, but only modified
+    as allowed by the permissions in ``allow``."""
+
+    R: Literal[2, 3, 4, 5, 6] = 6
+    """Select the security handler algorithm to use. Choose from:
+    ``2``, ``3``, ``4`` or ``6``. By default, the highest version of
+    is selected (``6``). ``5`` is a deprecated algorithm that should
+    not be used."""
+
+    allow: Permissions = DEFAULT_PERMISSIONS
+    """The permissions to set.
+    If omitted, all permissions are granted to the user."""
+
+    aes: bool = True
+    """If True, request the AES algorithm. If False, use RC4.
+    If omitted, AES is selected whenever possible (R >= 4)."""
+
+    metadata: bool = True
+    """If True, also encrypt the PDF metadata. If False,
+    metadata is not encrypted. Reading document metadata without
+    decryption may be desirable in some cases. Requires ``aes=True``.
+    If omitted, metadata is encrypted whenever possible."""
